@@ -2,6 +2,7 @@
 # this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 from collections import defaultdict
+from itertools import groupby
 
 from sql import Literal
 
@@ -11,8 +12,6 @@ from trytond.pyson import Eval, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 
-__all__ = ['Line', 'Move', 'MoveLine', 'OpenAccount']
-
 
 class Line(ModelSQL, ModelView):
     'Analytic Line'
@@ -43,18 +42,10 @@ class Line(ModelSQL, ModelView):
         super(Line, cls).__setup__()
         t = cls.__table__()
         cls._sql_constraints += [
-            ('credit_debit',
-                Check(t,
-                    (t.credit * t.debit == 0) & (t.credit + t.debit >= 0)),
-                'Wrong credit/debit values.'),
+            ('credit_debit_',
+                Check(t, t.credit * t.debit == 0),
+                'account.msg_line_debit_credit'),
             ]
-        cls._error_messages.update({
-                'line_on_view_account': (
-                    'You can not create a move line using '
-                    'view account "%s".'),
-                'line_on_inactive_account': ('You can not create a move line '
-                    'using inactive account "%s".'),
-                })
         cls._order.insert(0, ('date', 'ASC'))
 
     @classmethod
@@ -66,6 +57,9 @@ class Line(ModelSQL, ModelView):
         for field_name in ['name', 'journal']:
             table.not_null_action(field_name, action='remove')
 
+        # Migration from 5.0: replace credit_debit constraint by credit_debit_
+        table.drop_constraint('credit_debit')
+
     @staticmethod
     def default_date():
         Date = Pool().get('ir.date')
@@ -144,20 +138,6 @@ class Line(ModelSQL, ModelView):
         MoveLine.set_analytic_state(move_lines)
         MoveLine.save(move_lines)
 
-    @classmethod
-    def validate(cls, lines):
-        super(Line, cls).validate(lines)
-        for line in lines:
-            line.check_account()
-
-    def check_account(self):
-        if self.account.type == 'view':
-            self.raise_user_error('line_on_view_account',
-                (self.account.rec_name,))
-        if not self.account.active:
-            self.raise_user_error('line_on_inactive_account',
-                (self.account.rec_name,))
-
 
 class Move(metaclass=PoolMeta):
     __name__ = 'account.move'
@@ -200,6 +180,12 @@ class MoveLine(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(MoveLine, cls).__setup__()
+        cls._buttons.update({
+            'apply_analytic_rules': {
+                'invisible': Eval('analytic_state') != 'draft',
+                'depends': ['analytic_state'],
+                },
+            })
         cls._check_modify_exclude |= {'analytic_lines', 'analytic_state'}
 
     @classmethod
@@ -230,6 +216,18 @@ class MoveLine(ModelSQL, ModelView):
             'party': self.party.id if self.party else None,
             }
 
+    @property
+    def must_have_analytic(self):
+        "If the line must have analytic lines set"
+        pool = Pool()
+        FiscalYear = pool.get('account.fiscalyear')
+        if self.account.type:
+            return self.account.type.statement == 'income' and not (
+                # ignore balance move of non-deferral account
+                self.journal.type == 'situation'
+                and self.period.type == 'adjustment'
+                and isinstance(self.move.origin, FiscalYear))
+
     @classmethod
     def apply_rule(cls, lines):
         pool = Pool()
@@ -238,6 +236,8 @@ class MoveLine(ModelSQL, ModelView):
         rules = Rule.search([])
 
         for line in lines:
+            if not line.must_have_analytic:
+                continue
             if line.analytic_lines:
                 continue
             pattern = line.rule_pattern
@@ -249,30 +249,24 @@ class MoveLine(ModelSQL, ModelView):
             analytic_lines = []
             for entry in rule.analytic_accounts:
                 analytic_lines.extend(
-                    entry.get_analytic_lines(line, line.move.post_date))
+                    entry.get_analytic_lines(line, line.date))
             line.analytic_lines = analytic_lines
 
     @classmethod
     def set_analytic_state(cls, lines):
         pool = Pool()
-        AccountType = pool.get('account.account.type')
         AnalyticAccount = pool.get('analytic_account.account')
 
-        income_types = AccountType.search([
-                ('income_statement', '=', True),
-                ])
-        income_types = AccountType.search([
-                ('parent', 'child_of', [t.id for t in income_types]),
-                ])
-        income_types = set(income_types)
-
         roots = AnalyticAccount.search([
                 ('parent', '=', None),
-                ])
-        roots = set(roots)
+                ],
+            order=[('company', 'ASC')])
+        company2roots = {
+            company: set(roots)
+            for company, roots in groupby(roots, key=lambda r: r.company)}
 
         for line in lines:
-            if line.account.type not in income_types:
+            if not line.must_have_analytic:
                 if not line.analytic_lines:
                     line.analytic_state = 'valid'
                 else:
@@ -282,6 +276,7 @@ class MoveLine(ModelSQL, ModelView):
             for analytic_line in line.analytic_lines:
                 amount = analytic_line.debit - analytic_line.credit
                 amounts[analytic_line.account.root] += amount
+            roots = company2roots.get(line.move.company, set())
             if not roots <= set(amounts.keys()):
                 line.analytic_state = 'draft'
                 continue
@@ -293,6 +288,13 @@ class MoveLine(ModelSQL, ModelView):
             else:
                 line.analytic_state = 'valid'
 
+    @classmethod
+    @ModelView.button
+    def apply_analytic_rules(cls, lines):
+        cls.apply_rule(lines)
+        cls.set_analytic_state(lines)
+        cls.save(lines)
+
 
 class OpenAccount(Wizard):
     'Open Account'
@@ -302,7 +304,7 @@ class OpenAccount(Wizard):
 
     def do_open_(self, action):
         action['pyson_domain'] = [
-            ('account', '=', Transaction().context['active_id']),
+            ('account', '=', self.record.id if self.record else None),
             ]
         if Transaction().context.get('start_date'):
             action['pyson_domain'].append(
@@ -312,6 +314,8 @@ class OpenAccount(Wizard):
             action['pyson_domain'].append(
                 ('date', '<=', Transaction().context['end_date'])
                 )
+        if self.record:
+            action['name'] += ' (%s)' % self.record.rec_name
         action['pyson_domain'] = PYSONEncoder().encode(action['pyson_domain'])
         return action, {}
 
